#
# bim.R,  3 Feb 22
#
# Data from:
# https://www.projectdataanalytics.co.uk/calendar/event/39-projecthack13-delivering-projects-better
# Challenge 8

library("colorspace")
library("plyr")


par(bty="l")
par(las=1)
par(pch="+")

pal_col=rainbow(3)


# Plot number of drawings created per day (expressed as a percentage
# of project duration).
bim_per_day=function(proj_str)
{
p001=subset(dwg, Project == proj_str)

if (nrow(p001) < 50) # Require a minimum number of Clashes
   return(NULL)

# Metadata for this project
pmet=subset(meta, Project.Name == proj_str)

# Known percentage complection of project
perc_comp=as.numeric(gsub("%", "", pmet$X..Complete))

# Map Created date to percentage project completion
total_days=1+as.numeric(max(p001$Created)-min(p001$Created))
p001$perc=perc_comp*as.numeric(1+p001$Created-min(p001$Created))/total_days
created_cnt=count(p001$perc)
plot(created_cnt, log="y",
	xaxs="i",
	xlim=c(0, 100), ylim=c(1, max(created_cnt$freq, na.rm=TRUE)),
	xlab="Date (project percentage)", ylab="Clashes created")

mid_date=perc_comp/2 # min(p001$Created)+200
text(mid_date, max(created_cnt$freq)/2,
			label=paste0(proj_str, "\n", pmet$Contract.Type, "\n", pmet$Project.Performance))

# save plot to file having name of project
dev.copy(dev=pdf, file=paste0("drawings/", proj_str, ".pdf"))
dev.off()
}

# Plot number of drawings created per day (expressed as a percentage
# of project duration).
agg_bim_per_day=function(perf_str)
{
   # Map date to percentage project complete, rounded to nearest 5
   day_perc=function(df)
   {
   total_days=1+as.numeric(max(df$Created)-min(df$Created))
   # Round to nearest five
   df$perc=5*round((perc_comp*as.numeric(1+df$Created-min(df$Created))/total_days)/5)
   return(df)
   }

   contract_day=function(contract_str)
   {
   contr=subset(p_rnd, Contract.Type == contract_str)
   created_cnt=count(contr$perc)
   plot(created_cnt
	   xaxs="i",
	   xlim=c(0, 100), ylim=c(1, max(created_cnt$freq, na.rm=TRUE)),
	   xlab="Date (project percentage)", ylab="Drawings created")
   
   mid_date=perc_comp/2 # min(p001$Created)+200
   text(mid_date, max(created_cnt$freq)/2, label=paste0(perf_str, "\n", contract_str))
   
   # save plot to file having name of project
   dev.copy(dev=pdf, file=paste0("drawings/", perf_str, "_", contract_str, ".pdf"))
   dev.off()
   }

p_level=subset(dwg_m, Project.Performance == perf_str)

# Aggregate completed projects
p001=subset(p_level, X..Complete == "100%")

perc_comp=100 # as.numeric(gsub("%", "", pmet$X..Complete))

p_rnd=ddply(p001, .(Project), day_perc)

u_contract=unique(p_rnd$Contract.Type)
dum=sapply(u_contract, contract_day)

}

bim=read.csv("BIMCollab Data.csv", as.is=TRUE)
meta=read.csv("Project Metadata.csv", as.is=TRUE)
bim$Created=as.Date(bim$Created, format="%m/%d/%y")

bim=subset(bim, Status == "Closed")
bim=subset(bim, Type == "Clash")

# List of project names
u_proj=unique(bim$Project)

# bim_per_day("Project 024")

dum=sapply(u_proj, bim_per_day)

bim_m=merge(bim, meta, by.x="Project", by.y="Project.Name")

u_performance=unique(bim_m$Project.Performance)

dum=sapply(u_performance, agg_bim_per_day)

