#
# drawings.R,  5 Feb 22
#
# Data from:
# https://www.projectdataanalytics.co.uk/calendar/event/39-projecthack13-delivering-projects-better
# Challenge 8

library("colorspace")
library("plyr")


par(bty="l")
par(las=1)
par(pch="+")

pal_col=rainbow(3)


# Plot number of drawings created per day (expressed as a percentage
# of project duration).
draw_per_day=function(proj_str)
{
p001=subset(dwg, Project == proj_str)

if (nrow(p001) < 150) # Require a minimum number of project drawings
   return(NULL)

# Metadata for this project
pmet=subset(meta, Project.Name == proj_str)

# Known percentage complection of project
perc_comp=as.numeric(gsub("%", "", pmet$X..Complete))

# Map Created date to percentage project completion
total_days=1+as.numeric(max(p001$Created)-min(p001$Created))
p001$perc=perc_comp*as.numeric(1+p001$Created-min(p001$Created))/total_days
created_cnt=count(p001$perc)
construction=count(subset(p001, rev_status == "Construction")$perc)
design_dev=count(subset(p001, rev_status == "Design Development")$perc)
as_built=count(subset(p001, rev_status == "As Built")$perc)
plot(-1, type="n",
	xaxs="i",
	xlim=c(0, 100), ylim=c(1, max(created_cnt$freq, na.rm=TRUE)),
	xlab="Date (project percentage)", ylab="Drawings created")
points(as_built, type="b", col=pal_col[1])
points(construction, type="b", col=pal_col[2])
points(design_dev, type="b", col=pal_col[3])

legend("top", legend=c("As built", "Construction", "Design dev"), fill=pal_col)

mid_date=perc_comp/2 # min(p001$Created)+200
text(mid_date, max(created_cnt$freq)/2,
			label=paste0(proj_str, "\n", pmet$Contract.Type, "\n", pmet$Project.Performance))

# save plot to file having name of project
dev.copy(dev=pdf, file=paste0("drawings/", proj_str, ".pdf"))
dev.off()
}

# Plot number of drawings created per day (expressed as a percentage
# of project duration).
agg_draw_per_day=function(perf_str)
{
   # Map date to percentage project complete, rounded to nearest 5
   day_perc=function(df)
   {
   total_days=1+as.numeric(max(df$Created)-min(df$Created))
   # Round to nearest five
   df$perc=5*round((perc_comp*as.numeric(1+df$Created-min(df$Created))/total_days)/5)
   return(df)
   }

   contract_day=function(contract_str)
   {
   contr=subset(p_rnd, Contract.Type == contract_str)
   created_cnt=count(contr$perc)
   construction=count(subset(contr, rev_status == "Construction")$perc)
   design_dev=count(subset(contr, rev_status == "Design Development")$perc)
   as_built=count(subset(contr, rev_status == "As Built")$perc)
   plot(-1, type="n",
	   xaxs="i",
	   xlim=c(0, 100), ylim=c(1, max(created_cnt$freq, na.rm=TRUE)),
	   xlab="Date (project percentage)", ylab="Drawings created")
   points(as_built, type="b", col=pal_col[1])
   points(construction, type="b", col=pal_col[2])
   points(design_dev, type="b", col=pal_col[3])
   
   legend("top", legend=c("As built", "Construction", "Design dev"), fill=pal_col)
   
   mid_date=perc_comp/2 # min(p001$Created)+200
   text(mid_date, max(created_cnt$freq)/2, label=paste0(perf_str, "\n", contract_str))
   
   # save plot to file having name of project
   dev.copy(dev=pdf, file=paste0("drawings/", perf_str, "_", contract_str, ".pdf"))
   dev.off()
   }

p_level=subset(dwg_m, Project.Performance == perf_str)

# Aggregate completed projects
p001=subset(p_level, X..Complete == "100%")

perc_comp=100 # as.numeric(gsub("%", "", pmet$X..Complete))

p_rnd=ddply(p001, .(Project), day_perc)

u_contract=unique(p_rnd$Contract.Type)
dum=sapply(u_contract, contract_day)

}

dwg=read.csv("Combined DWG Data.csv", as.is=TRUE)
map=read.csv("map_table.csv", as.is=TRUE)
meta=read.csv("Project Metadata.csv", as.is=TRUE)
dwg=read.csv("Combined DWG Data.csv", as.is=TRUE)
dwg$Created=as.Date(dwg$Created.Date, format="%m/%d/%y")
dwg$Revision=as.Date(dwg$Revision.Authored.Date, format="%m/%d/%y")
dwg$Due=as.Date(dwg$Due.Date, format="%m/%d/%y")

# Number of days between Created, Revision and Due dates
dwg$CR_days=dwg$Revision-dwg$Created
dwg$CD_days=dwg$Due-dwg$Created
dwg$RD_days=dwg$Due-dwg$Revision

dwg$rev_status=mapvalues(dwg$Revision.Status, map$All.Revision.Status, map$Construction)

# CR_cnt=count(dwg$CR_days)
# plot(CR_cnt, log="xy",
# 	xlab="Days (revision)", ylab="Drawings")

created_cnt=count(dwg$Created)
# plot(created_cnt)

CD_cnt=count(dwg$CD_days)
# plot(CD_cnt, log="xy",
# 	# xlim=c(1, 2500), ylim=c(1, 2000),
# 	xlab="Days (due)", ylab="Occurrences")

RD_cnt=count(dwg$RD_days)
# plot(RD_cnt, log="xy",
# 	# xlim=c(1, 2500), ylim=c(1, 2000),
# 	xlab="Days (due)", ylab="Occurrences")

# List of project names
u_proj=unique(dwg$Project)

# draw_per_day("Project 001")

dum=sapply(u_proj, draw_per_day)

dwg_m=merge(dwg, meta, by.x="Project", by.y="Project.Name")

u_performance=unique(dwg_m$Project.Performance)

dum=sapply(u_performance, agg_draw_per_day)

