#
# rfi.R,  4 Feb 22
#
# Data from:
# https://www.projectdataanalytics.co.uk/calendar/event/39-projecthack13-delivering-projects-better
# Challenge 8

library("colorspace")
library("plyr")


par(bty="l")
par(las=1)
par(pch="+")

pal_col=rainbow(3)


# Number of RFIs taking a given number of days to resolve, by Project
rfi_days=function(proj_str)
{
p001=subset(rfi, Project.PSD == proj_str)

if (nrow(p001) < 150)
   return(NULL)

pday=ddply(p001, .(RFI.Name.PSD, PriorityDescription),
		function(df) return(max(df$Authored)-min(df$Authored)))
high=subset(pday, PriorityDescription == "High")
medium=subset(pday, PriorityDescription == "Medium")
low=subset(pday, PriorityDescription == "Low")

dur_count=count(pday$V1)
high_count=count(high$V1)
medium_count=count(medium$V1)
low_count=count(low$V1)
plot(dur_count, type="n", log="y",
	xlab="Duration (days)", ylab="RFIs")

text(50, max(dur_count$freq)/2, proj_str)

points(high_count, col=pal_col[1])
points(medium_count, col=pal_col[2])
points(low_count, col=pal_col[3])

legend("topright", legend=c("High", "Medium", "Low"), fill=pal_col)

dev.copy(dev=pdf, file=paste0("rfi/", proj_str, ".pdf"))
dev.off()
}


# Number of RFIs taking a given number of days to resolve, by Project
perf_rfi_days=function(perf_str)
{
   rfi_dur=function(df_dur)
   {
   pday=ddply(df_dur, .(RFI.Name.PSD, PriorityDescription),
		function(df) return(10*round((max(df$Authored)-min(df$Authored))/10)))
   return(pday)
   }

   cont_days=function(cont_str)
   {
   c_df=subset(p001, Contract.Type == cont_str)

   c_days=rfi_dur(c_df)
   high=subset(c_days, PriorityDescription == "High")
   medium=subset(c_days, PriorityDescription == "Medium")
   low=subset(c_days, PriorityDescription == "Low")

   dur_count=count(c_days$V1)
   high_count=count(high$V1)
   medium_count=count(medium$V1)
   low_count=count(low$V1)
   plot(dur_count, type="n", log="y",
	   xlab="Duration (days)", ylab="RFIs")

   text(200, max(dur_count$freq)/2, paste0(perf_str, "\n", cont_str))

   points(high_count, col=pal_col[1])
   points(medium_count, col=pal_col[2])
   points(low_count, col=pal_col[3])

   legend("topright", legend=c("High", "Medium", "Low"), fill=pal_col)

   dev.copy(dev=pdf, file=paste0("rfi/", perf_str, ",", cont_str, ".pdf"))
   dev.off()
   }

p001=subset(rfi_m, Project.Performance == perf_str)

u_cont=unique(p001$Contract.Type)
dum=sapply(u_cont, cont_days)

}


meta=read.csv("Project Metadata.csv", as.is=TRUE)
rfi=read.csv("Combined RFI Data.csv", as.is=TRUE)
rfi$Authored=as.Date(rfi$Authored.Date, format="%d %B %Y")
dotsep=regexpr(".", rfi$Revision, fixed=TRUE) # offset of . in Revsion
rfi$subversion=as.numeric(substring(rfi$Revision, dotsep+1))


plot(count(rfi$subversion), log="y")

u_proj=unique(rfi$Project.PSD)

rfi_days("Project 001")

dum=sapply(u_proj, rfi_days)

rfi_m=merge(rfi, meta, by.x="Project.PSD", by.y="Project.Name")

u_perf=unique(rfi_m$Project.Performance)

dum=sapply(u_perf, perf_rfi_days)

